from .Command import Command
from .Pipe import Pipe
from .PipeBuilder import PipeBuilder

class PipeBuilderConcrete(PipeBuilder):

    def __init__(self):
        self.pipe = Pipe()
    
    def build(self) -> Pipe:
        return self.pipe

    def command(self, command: Command):
        self.pipe.commands.append(command)
        return self


from .Command import Command
from .Sink import Sink
from multiprocessing.pool import ThreadPool

class Pipe(Command):

    def __init__(self):
        self.pool = ThreadPool()
        self.commands = list()

    def parallelize(self, sink: Sink, command: Command):
        arguments = [(sink, )]
        sink.collect(self.pool.starmap(command.do, arguments)[0])

    def do(self, sink: Sink):
        for command in self.commands:
            self.parallelize(sink, command)


from .Command import Command
from .Sink import Sink

class Sum(Command):

    def __init__(self, amount):
        self.amount = amount

    def sum(self, sink: Sink):
        return [i + self.amount for i in sink.data]

    def do(self, sink: Sink):
        return self.sum(sink)


import abc
from .Command import Command
from .Pipe import Pipe

class PipeBuilder(abc.ABC):
    
    @abc.abstractproperty
    def build(self) -> Pipe:
        pass

    @abc.abstractmethod
    def command(self, command: Command):
        pass


from easypype.Command import Command
from easypype.ConcreteSink import ConcreteSink
from easypype.Pipe import Pipe
from easypype.PipeBuilder import PipeBuilder
from easypype.PipeBuilderConcrete import PipeBuilderConcrete
from easypype.Sink import Sink
from easypype.Sum import Sum